﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using System.Drawing.Imaging;

namespace YUV_Player
{
    public class Picture
    {
        /// <summary>
        /// YUV 스트림을 읽어오기 위한 변수
        /// </summary>
        byte[] _buf;
        public byte[] Buf
        {
            get { return _buf; }
            set { _buf = value; }
        }

        /// <summary>
        /// RGB로 변경된 Bitmap들
        /// </summary>
        Bitmap[] _movie;
        public Bitmap[] Movie
        {
            get { return _movie; }
            private set { _movie = value; }
        }

        /// <summary>
        /// YUV 시퀀스 정보
        /// </summary>
        PictureInfomation _pictureInfo;
        public PictureInfomation PictureInfo
        {
            get { return _pictureInfo; }
            set { _pictureInfo = value; }
        }        

        public Picture(PictureInfomation _pictureInfo)
        {
            this.PictureInfo = _pictureInfo;
            ReadYUVFile(PictureInfo.FilePath);
        }

        /// <summary>
        /// YUV 파일을 읽어온다.
        /// </summary>
        /// <param name="filePath"></param>
        private void ReadYUVFile(string filePath)
        {
            Color[] data;
            //해당 경로에 파일이 있는지 확인
            if (File.Exists(filePath))
            {
                using (FileStream stream = File.OpenRead(filePath))
                {
                    using (BinaryReader br = new BinaryReader(stream))
                    {
                        Buf = br.ReadBytes((int)stream.Length);
                        data = new Color[(int)stream.Length];
                    }
                }
                Movie = new Bitmap[Buf.Length / (PictureInfo.frameSize + (PictureInfo.frameSize / 2))];
            }
            else
            {
                MessageBox.Show( "파일을 열 수 없습니다. 파일을 확인하고 다시 시도 하세요.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// 해당하는 프레임을 RGB로 변환하여 Bitmap으로 만든다.
        /// </summary>
        /// <param name="frameNum">만들 프레임 번호</param>
        /// <returns></returns>
        public Bitmap MakeFrame(int frameNum)
        {
            if (Movie[frameNum] != null) return Movie[frameNum];

            Bitmap bitmap = new Bitmap(PictureInfo.Size.Width, PictureInfo.Size.Height, PixelFormat.Format24bppRgb);
            int i, cnt, uIndex, vIndex, frameSeek;

            for (i = 0, cnt = 0, uIndex = PictureInfo.frameSize, vIndex = uIndex + (PictureInfo.frameSize / 4); i < PictureInfo.frameSize; i += 2, cnt += 2, uIndex++, vIndex++)
            {
                BitmapData d = new BitmapData();

                frameSeek = (PictureInfo.frameSize + (PictureInfo.frameSize / 2)) * frameNum; ;
                bitmap.SetPixel(i % PictureInfo.Size.Width, i / PictureInfo.Size.Width,
                    YUVtoRGB(Buf[i + frameSeek], Buf[uIndex + frameSeek], Buf[vIndex + frameSeek]));

                bitmap.SetPixel((i + 1) % PictureInfo.Size.Width, (i + 1) / PictureInfo.Size.Width,
                    YUVtoRGB(Buf[i + 1 + frameSeek], Buf[uIndex + frameSeek], Buf[vIndex + frameSeek]));

                bitmap.SetPixel((i + PictureInfo.Size.Width) % PictureInfo.Size.Width, (i + PictureInfo.Size.Width) / PictureInfo.Size.Width,
                    YUVtoRGB(Buf[i + PictureInfo.Size.Width + frameSeek], Buf[uIndex + frameSeek], Buf[vIndex + frameSeek]));

                bitmap.SetPixel((i + PictureInfo.Size.Width + 1) % PictureInfo.Size.Width, (i + PictureInfo.Size.Width + 1) / PictureInfo.Size.Width,
                    YUVtoRGB(Buf[i + PictureInfo.Size.Width + 1 + frameSeek], Buf[uIndex + frameSeek], Buf[vIndex + frameSeek]));
                if (cnt == PictureInfo.Size.Width - 2)
                {
                    cnt = -2;
                    i += PictureInfo.Size.Width;
                }
            }
            Movie[frameNum] = bitmap;
            return bitmap;
        }

        /// <summary>
        /// YUV포맷을 RGB로 변환
        /// </summary>
        /// <param name="Y"></param>
        /// <param name="U"></param>
        /// <param name="V"></param>
        /// <returns></returns>
        public Color YUVtoRGB(int Y, int U, int V)
        {
            return Color.FromArgb(Clippiing((76284 * (Y - 16) + 104595 * (V - 128)) >> 16),
                Clippiing((76284 * (Y - 16) - 53281 * (V - 128) - 25625 * (U - 128)) >> 16),
                Clippiing((76284 * (Y - 16) + 132252 * (U - 128)) >> 16));
        }

        /// <summary>
        /// 인자로 들어온 값을 0~255 사이인지 확인하여 
        /// 넘었을 경우 0이나 255로 바꾼다.
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        byte Clippiing(int data)
        {
            return (byte)((data < 0) ? 0 : ((data > 255) ? 255 : data));
        }
    }
}
